#include <avr/wdt.h>

const int POWER_OUTAGE_INPUT = A0;
const int LOW_BATTERY_INPUT = A1;
const int SHUTDOWN_OUTPUT = A2;
const int NAS_STILL_RUNNING_INPUT = PD7;
const boolean KEEP_UPS_RUNNING = LOW;
const boolean SHUT_OFF_UPS = HIGH;
// using print(\n) instead of println( ), we avoid sending superfluous linefeed
const String OK_REPLY_FOR_NAS = "OK\n";
const String SHUTDOWN_COMMAND_FOR_NAS = "NASShutdown\n";

const int POWER_OK_NOT_TIMING=0;
const unsigned long MAX_SECONDS_OF_CONTINUOUS_OUTAGE = 10;
const unsigned long MILLIS_PER_SECOND = 1000;
const unsigned long NAS_POLLING_INTERVAL_MS = 1000;


void setup(void) {
  pinMode(LED_BUILTIN, OUTPUT);

  pinMode(POWER_OUTAGE_INPUT, INPUT_PULLUP);
  pinMode(LOW_BATTERY_INPUT, INPUT_PULLUP);
  pinMode(NAS_STILL_RUNNING_INPUT, INPUT_PULLUP);
  
  pinMode(SHUTDOWN_OUTPUT, OUTPUT);

  digitalWrite(SHUTDOWN_OUTPUT, KEEP_UPS_RUNNING);
  digitalWrite(LED_BUILTIN, HIGH); 

  // using default timeout (1000 ms)
  Serial.begin(9600, SERIAL_8N1);

  // enable the watchdog
  wdt_enable(WDTO_8S);     
}


void shutdownUPS() 
{
  wdt_reset();

  digitalWrite(SHUTDOWN_OUTPUT, SHUT_OFF_UPS);
  delay(1000);  // minimum 500 ms for APC UPS
  // UPS will be off now
  digitalWrite(SHUTDOWN_OUTPUT, KEEP_UPS_RUNNING);
}


void loop(void) {
  unsigned long iPowerOutBeginTime=POWER_OK_NOT_TIMING;
  
  do {
    wdt_reset();

    // from UPS
    boolean bBatteryOK = digitalRead(LOW_BATTERY_INPUT);
    boolean bPowerOK = digitalRead(POWER_OUTAGE_INPUT);

    if (bPowerOK) {
      iPowerOutBeginTime = POWER_OK_NOT_TIMING;
    } else {
      if (iPowerOutBeginTime==POWER_OK_NOT_TIMING) 
        iPowerOutBeginTime = millis(); 

      // when NAS shutdown is complete, USB power will disappear; we respond to that
      if (!digitalRead(NAS_STILL_RUNNING_INPUT))
        shutdownUPS();
    }

    if (Serial.available()>0) {
      // if it "times out", command will be dropped
      // resulting string excludes the found "ending"
      String s = Serial.readStringUntil('\n');

      //Serial.print("Received:  ");
      //Serial.println(s);
      
      if (s.indexOf("HowAreThings?")>=0) {
//        digitalWrite(LED_BUILTIN, LOW); 
//        delay(200);
//        digitalWrite(LED_BUILTIN, HIGH); 

        if (iPowerOutBeginTime == POWER_OK_NOT_TIMING)
          // constant includes \n
          Serial.print(OK_REPLY_FOR_NAS);
          else
          // it's an acceptable risk that the power outage begins around the millis rollover (every 50 days).
          // immediately commence shutdown if battery low
          if ((millis() - iPowerOutBeginTime > MAX_SECONDS_OF_CONTINUOUS_OUTAGE * MILLIS_PER_SECOND) || (!bBatteryOK)) 
            // constant includes \n
            Serial.print(SHUTDOWN_COMMAND_FOR_NAS);
            else
            // constant includes \n
            Serial.print(OK_REPLY_FOR_NAS);
      }
    }
    
    // handy for debugging 
//    Serial.print(bPowerOK? "PowerOK  " : "PowerOut  ");
//    Serial.print(bBatteryOK? "BatteryOK" : "LowBattery");
//    // using print(\n) instead of println( ), we avoid sending superfluous linefeed
//    Serial.print("\n");

    wdt_reset();

    delay(NAS_POLLING_INTERVAL_MS);
  } while (true);
}


